(* $Id: miniui.mli 678 2007-02-25 17:16:54Z gerd $
 * ----------------------------------------------------------------------
 * Copyright 2003 by Gerd Stolpmann. 
 * 
 * This file is part of godi-tools. 
 *
 * godi-tools is free software; you can redistribute it and/or modify it
 * under the conditions in the file LICENSE.
 *
 * godi-tools is distributed WITHOUT ANY WARRANTY, as detailed in the
 * file LICENSE.
 *)

(* Miniui is an abstract description of a simple user interface system.
 *
 * A page is a grid of characters (all have the same width). It has three
 * regions: Head, body, foot. The head region is displayed in the upper
 * part of the page, the foot in the lower part, and the body in the
 * middle.
 *
 * In the head and body regions arbitrary text can be displayed. The
 * body region is scrollable.
 *
 * The foot region is reserved for buttons, and text input.
 *
 * The page is actually drawn on the screen when the [update] method is 
 * called, or automatically triggered by the event [loop].
 *
 * Events are processed by the event [loop], or when [poll_event] is
 * called. The latter returns immediately if there is no pending event.
 *)

type region = [ `Head | `Body ];;

type line_spec = [ `Replace of int | `Append ];;

type key_spec = [ `Keypress of char | `Number of int ];;


class type page =
object('self)
  method configure : ?head_region_height:int ->
                     ?auto_scroll:bool ->
                     unit ->
		       unit
    (* Configure global properties of the page:
     * ~head_region_height: how many lines the head region has 
     * ~auto_scroll: whether the body scrolls automatically when new lines
     *    are added to its end
     *)

  method width : unit -> int
    (* Returns the width of the page *)

  method height : ?region:region -> unit -> int
    (* Returns the height of the region. ~region defaults to `Body *)

  method clear : unit -> unit
    (* Clears everything from the page *)

  method put_line : ?region:region ->
                    ?line:line_spec ->
                    ?lmargin:int ->
		    ?rmargin:int ->
		    string ->
		      unit
    (* Replace or add the line identified by ~line of ~region.
     * ~region: defaults to `Body
     * ~lmargin: the margin at the left side
     * ~rmargin: the margin at the right side
     *)

  method get_line : ?region:region -> int -> string
    (* Return the contents of the line of the ~region. Returns "" if the
     * line has never been written.
     * ~region: defaults to `Body
     *)

  method length : ?region:region -> unit -> int
    (* Returns the number of lines of the region *)

  method add_body_button : line:int ->
                           ?left:int ->
                           ?width:int ->
			   ?callback:(unit -> unit) ->
                           key_spec ->
			     button_id
    (* A body button is a sensitive region within a line (think of 
     * hyperlinks in web browsers). The region begins at character
     * position ~left, and has a ~width. It has also a keyboard binding.
     * The ~callback is invoked when the body button is pressed.
     *)

  method add_foot_button : ?text:string ->
                           ?underline:int ->
                           ?callback:(unit -> unit) ->
                           key_spec ->
			     button_id
    (* A foot button is placed into the foot region. It has a ~text,
     * and one character of the text can be emphasized (underlined).
     * Furthermore, it has a keyboard binding.
     * Without ~text, only the keyboard binding is active, the button
     * is invisible.
     *)

  method remove_button : button_id -> unit
    (* Remove the identified button *)

  method activate_text_entry : ?value:string ->
                               ?prompt:string ->
                               ?callback_ok:(string -> unit) ->
			       ?callback_cancel:(unit -> unit) ->
			       unit ->
				 unit
    (* The text entry box has a prompt, and a value that can be edited.
     * The user can confirm the input, in which case ~callback_ok is
     * invoked, or he can cancel the input, in which case ~callback_cancel
     * is called.
     *)

  method set_busy : bool -> unit
    (* Sets the busy flag. If set, it is indicated to the user that input
     * is not accepted (e.g. "hourglass cursor").
     *)

  method busy : bool
    (* Queries the busy flag *)

  method set_scroll_position : float -> unit

  method scroll_position : float
    (* The scroll position is a number from 0.0 to 1.0 *)

  method update : ?force:bool -> unit -> unit
    (* Updates the display *)

  method poll_event : unit -> unit
    (* Checks whether there is a pending event, and executes the associated
     * handlers.
     * This method returns immediately after that.
     *)

  method quit : unit -> unit
    (* If called from a callback, this method causes that [loop] returns
     * to its caller.
     *)

  method loop : unit -> unit
    (* Repeatedly update the screen, and poll for new events. *)

  method close : unit -> unit
    (* Closes the graphical window, if any *)

  method new_page : unit -> page
    (* Create a new page. The first time the [update] or [loop] of the
     * new page are called the current page is closed.
     *)

end

and button_id = object end
;;
